@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
  * libdatrie - Double-Array Trie Library
- * Copyright (C) 2006  Theppitak Karoonboonyanan <thep@linux.thai.net>
+ * Copyright (C) 2006  Theppitak Karoonboonyanan <theppitak@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
 /*
  * darray.c - Double-array trie structure
  * Created: 2006-08-13
- * Author:  Theppitak Karoonboonyanan <thep@linux.thai.net>
+ * Author:  Theppitak Karoonboonyanan <theppitak@gmail.com>
  */
 
 #include <string.h>
@@ -41,9 +41,11 @@
 
 struct _Symbols {
     short       num_symbols;
-    TrieChar    symbols[256];
+    TrieChar    symbols[TRIE_CHAR_MAX + 1];
 };
 
+static Symbols *    symbols_new (void);
+static void         symbols_add (Symbols *syms, TrieChar c);
 #define symbols_add_fast(s,c)   ((s)->symbols[(s)->num_symbols++] = c)
 
 /*-----------------------------------*
@@ -84,14 +86,14 @@ static void         da_free_cell       (DArray         *d,
  *   INTERNAL TYPES IMPLEMENTATIONS   *
  *------------------------------------*/
 
-Symbols *
-symbols_new ()
+static Symbols *
+symbols_new (void)
 {
     Symbols *syms;
 
     syms = (Symbols *) malloc (sizeof (Symbols));
 
-    if (!syms)
+    if (UNLIKELY (!syms))
         return NULL;
 
     syms->num_symbols = 0;
@@ -105,7 +107,7 @@ symbols_free (Symbols *syms)
     free (syms);
 }
 
-void
+static void
 symbols_add (Symbols *syms, TrieChar c)
 {
     short lower, upper;
@@ -178,17 +180,17 @@ struct _DArray {
  * Create a new empty doubla-array object.
  */
 DArray *
-da_new ()
+da_new (void)
 {
     DArray     *d;
 
     d = (DArray *) malloc (sizeof (DArray));
-    if (!d)
+    if (UNLIKELY (!d))
         return NULL;
 
     d->num_cells = DA_POOL_BEGIN;
     d->cells     = (DACell *) malloc (d->num_cells * sizeof (DACell));
-    if (!d->cells)
+    if (UNLIKELY (!d->cells))
         goto exit_da_created;
     d->cells[0].base = DA_SIGNATURE;
     d->cells[0].check = d->num_cells;
@@ -227,7 +229,8 @@ da_fread (FILE *file)
     if (!file_read_int32 (file, &n) || DA_SIGNATURE != (uint32) n)
         goto exit_file_read;
 
-    if (NULL == (d = (DArray *) malloc (sizeof (DArray))))
+    d = (DArray *) malloc (sizeof (DArray));
+    if (UNLIKELY (!d))
         goto exit_file_read;
 
     /* read number of cells */
@@ -236,7 +239,7 @@ da_fread (FILE *file)
     if (d->num_cells > SIZE_MAX / sizeof (DACell))
         goto exit_da_created;
     d->cells = (DACell *) malloc (d->num_cells * sizeof (DACell));
-    if (!d->cells)
+    if (UNLIKELY (!d->cells))
         goto exit_da_created;
     d->cells[0].base = DA_SIGNATURE;
     d->cells[0].check= d->num_cells;
@@ -332,7 +335,7 @@ da_get_root (const DArray *d)
 TrieIndex
 da_get_base (const DArray *d, TrieIndex s)
 {
-    return (s < d->num_cells) ? d->cells[s].base : TRIE_INDEX_ERROR;
+    return LIKELY (s < d->num_cells) ? d->cells[s].base : TRIE_INDEX_ERROR;
 }
 
 /**
@@ -348,7 +351,7 @@ da_get_base (const DArray *d, TrieIndex s)
 TrieIndex
 da_get_check (const DArray *d, TrieIndex s)
 {
-    return (s < d->num_cells) ? d->cells[s].check : TRIE_INDEX_ERROR;
+    return LIKELY (s < d->num_cells) ? d->cells[s].check : TRIE_INDEX_ERROR;
 }
 
 
@@ -364,7 +367,7 @@ da_get_check (const DArray *d, TrieIndex s)
 void
 da_set_base (DArray *d, TrieIndex s, TrieIndex val)
 {
-    if (s < d->num_cells) {
+    if (LIKELY (s < d->num_cells)) {
         d->cells[s].base = val;
     }
 }
@@ -381,7 +384,7 @@ da_set_base (DArray *d, TrieIndex s, TrieIndex val)
 void
 da_set_check (DArray *d, TrieIndex s, TrieIndex val)
 {
-    if (s < d->num_cells) {
+    if (LIKELY (s < d->num_cells)) {
         d->cells[s].check = val;
     }
 }
@@ -453,7 +456,7 @@ da_insert_branch (DArray *d, TrieIndex s, TrieChar c)
             new_base = da_find_free_base (d, symbols);
             symbols_free (symbols);
 
-            if (TRIE_INDEX_ERROR == new_base)
+            if (UNLIKELY (TRIE_INDEX_ERROR == new_base))
                 return TRIE_INDEX_ERROR;
 
             da_relocate_base (d, s, new_base);
@@ -468,7 +471,7 @@ da_insert_branch (DArray *d, TrieIndex s, TrieChar c)
         new_base = da_find_free_base (d, symbols);
         symbols_free (symbols);
 
-        if (TRIE_INDEX_ERROR == new_base)
+        if (UNLIKELY (TRIE_INDEX_ERROR == new_base))
             return TRIE_INDEX_ERROR;
 
         da_set_base (d, s, new_base);
@@ -555,7 +558,7 @@ da_find_free_base  (DArray         *d,
     while (!da_fit_symbols (d, s - first_sym, symbols)) {
         /* extend pool before getting exhausted */
         if (-da_get_check (d, s) == da_get_free_list (d)) {
-            if (!da_extend_pool (d, d->num_cells))
+            if (UNLIKELY (!da_extend_pool (d, d->num_cells)))
                 return TRIE_INDEX_ERROR;
         }
 
@@ -637,17 +640,22 @@ static Bool
 da_extend_pool     (DArray         *d,
                     TrieIndex       to_index)
 {
+    void       *new_block;
     TrieIndex   new_begin;
     TrieIndex   i;
     TrieIndex   free_tail;
 
-    if (to_index <= 0 || TRIE_INDEX_MAX <= to_index)
+    if (UNLIKELY (to_index <= 0 || TRIE_INDEX_MAX <= to_index))
         return FALSE;
 
     if (to_index < d->num_cells)
         return TRUE;
 
-    d->cells = (DACell *) realloc (d->cells, (to_index + 1) * sizeof (DACell));
+    new_block = realloc (d->cells, (to_index + 1) * sizeof (DACell));
+    if (UNLIKELY (!new_block))
+        return FALSE;
+
+    d->cells = (DACell *) new_block;
     new_begin = d->num_cells;
     d->num_cells = to_index + 1;
 
@@ -774,7 +782,7 @@ da_first_separate (DArray *d, TrieIndex root, TrieString *keybuff)
                 break;
         }
 
-        if (c == max_c)
+        if (c > max_c)
             return TRIE_INDEX_ERROR;
 
         trie_string_append_char (keybuff, c);
